#include <CompactQik2s9v1.h>

#include <SoftwareSerial.h>

#define rxPin 5
#define txPin 6
#define rstPin 7

 
SoftwareSerial mySerial =  SoftwareSerial(rxPin, txPin);
CompactQik2s9v1 motor = CompactQik2s9v1(&mySerial,rstPin);

int stopAt = 40;            // Go 40 "ticks" and stop
byte normalSpeed = 127;  // Regular speed
byte slowSpeed = 100;    // Governed speed

const int encRight = 2; 
const int encLeft = 3;
int countRight = 0;
int countLeft = 0;
volatile int triggerLeft = 0;
volatile int triggerRight = 0;

void setup() {
  Serial.begin(115200);

  // Set pin modes
  pinMode(encLeft, INPUT);
  pinMode(encRight, INPUT);

  // Set up interrupts
  attachInterrupt(0, hitRight, RISING);  // Pin 2
  attachInterrupt(1, hitLeft, RISING);   // Pin 3

  mySerial.begin(9600);
  motor.begin();
  motor.stopBothMotors();
  delay(200);
  motor.motor0Forward(normalSpeed);    // Left motor
  motor.motor1Forward(normalSpeed);    // Right motor
}

void loop() {
  if (countLeft > stopAt) {
    motor.motor0Forward(0);
  }

  if (countRight > stopAt) {
    motor.motor1Forward(0);
  }

  if (countLeft >= stopAt && countRight >= stopAt) {
    motor.motor0Forward(0);
    motor.motor1Forward(0);
    return;
  }

  if (countRight >= countLeft) {
    motor.motor0Forward(slowSpeed);
    delayMicroseconds(15);
    motor.motor0Forward(normalSpeed);  
  } else {
    motor.motor1Forward(slowSpeed);
    delayMicroseconds(15);
    motor.motor1Forward(normalSpeed);
  }

  // If left encoder hit, show results
  if (triggerLeft == HIGH) { 
    Serial.print("L: ");
    Serial.println(countLeft, DEC);
    triggerLeft = LOW;
  }

  // If right encoder hit, show results
  if (triggerRight == HIGH) {
    Serial.print("R: ");  
    Serial.println(countRight, DEC);
    triggerRight = LOW;
  }
}

// Interrupt handlers
void hitLeft() {
  static unsigned long last_interrupt_timeL = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_timeL > 15) {
    triggerLeft = HIGH;
    countLeft++;
  }
  last_interrupt_timeL = interrupt_time;
}

void hitRight() {
  static unsigned long last_interrupt_timeR = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_timeR > 15) {
    triggerRight = HIGH;
    countRight++;
  }
  last_interrupt_timeR = interrupt_time;
}

